#!/usr/bin/python

"""
HW8 for Stanford's CS224U: Natural Language Understanding
Winter 2012
Bill McCartney and Christopher Potts

This program uses the nltk.sem to define a first-order model, and it
uses basic CGI functionality to provide a query interface to that
model.

The associated file template.html provides the HTML code.
"""

import cgi
import cgitb
cgitb.enable()

print "Content-Type: text/html"
print

import os
from glob import glob
import cPickle as pickle
import nltk

######################################################################

TEMPLATE = 'template.html'

# For the live version, we pickled the output of
# define_model_and_assignment() for faster loading:
MODEL_PICKLE_FILENAME = 'model_assignment_tuple.pickle'

######################################################################

def define_model_and_assignment():
    """Defines the model M and the assignment function g."""
    # Domain of entities:
    dom = set(['Bart', 'Lisa', 'Maggie', 'Homer', 'Marge', 'Burns'])
    # The valuation is specified in this string format and then converted
    # to an Valuation object by nltk.parse_valuation():    
    val_str = """
bart => Bart
lisa => Lisa
maggie => Maggie
homer => Homer
marge => Marge
burns => Burns
simpson => {Bart, Lisa, Maggie, Homer, Marge}
child => {Bart, Lisa, Maggie}
parent => {Homer, Marge}
boss => {Burns}
female => {Lisa, Maggie, Marge}
male => {Bart, Homer, Burns}
bald => {Homer, Burns}
musical => {Lisa, Maggie}
student => {Lisa, Bart}
skateboarder => {Bart, Homer}
teases => {(Bart, Lisa), (Lisa, Bart), (Bart, Homer)}
admires => {(Bart, Lisa), (Lisa, Lisa), (Maggie, Lisa), (Homer, Lisa), (Marge, Lisa), (Burns, Lisa)}
telephoned => {(Bart, Lisa), (Lisa, Bart), (Maggie, Burns), (Homer, Marge), (Marge, Homer), (Burns, Burns)}
"""
    # Create the Valuation:
    val = nltk.parse_valuation(val_str)
    # Create the Model:
    mod = nltk.Model(dom, val)
    # Create an assignment function (individual-level variables only):
    g = nltk.Assignment(dom, [('x', 'Maggie'), ('y', 'Lisa'), ('z', 'Bart')])
    return (mod, g)

# Define the model and assignment function:
mod, g = define_model_and_assignment()

# Faster to load the model and assignment from a pickle:
# pickle.dump((mod, g), file('model_assignment_tuple.pickle', 'w'), 2)
# mod, g = pickle.load(file('model_assignment_tuple.pickle'))

def main():
    """
    Controls the flow: if the cgi.FieldStorage instance has a value
    for 'formula', then we try to interpret that formula, otherwise we
    just display the assignment with its embedded query.
    
    """
    form = cgi.FieldStorage()
    formula = get_value(form, 'formula')
    interpretation = ''
    if formula:
        # Catch anything we can't interpret and blame it on the user!
        try:
            # The assignment function g is a parameter on
            # interpretation, needed wherever there are variables.
            interpretation = str(mod.evaluate(formula, g))
        except:
            interpretation = "We weren't able to parse your formula.  Check to make sure it is well-formed."
    display(formula, interpretation)

def get_value(form, key):
    """
    Get the value for key in the cgi.FieldStorage instance
    form. Return the emptry string if there is no such key.
    """
    if key in form and form.getfirst(key):
        return form.getvalue(key)
    else:
        return ''

def display(formula, interpretation):
    """
    Reads in the template HTML file, inserts the formual and
    interpretation strings (which might be empty) and print the result
    to standard output, i.e., the browser window.
    """
    contents = open(TEMPLATE, 'r').read()
    contents = contents.replace('{{ formula }}', formula)
    contents = contents.replace('{{ interpretation }}', interpretation)
    print contents


main()

